#include "pch.h"
#include "common.h"
#include "InputManager.h"

#include <gainput/gainput.h>

#ifdef _DEBUG
#pragma comment(lib, "gainputstatic-d.lib")
#else
#pragma comment(lib, "gainputstatic.lib")
#endif

#pragma comment(lib, "XInput.lib")
#pragma comment(lib, "Ws2_32.lib")

InputManager* g_InputManager;

static gainput::InputManager* g_GaInputManager;
static gainput::InputMap* g_GaInputMap;

static const uint32_t PAD_0_BASE_INDEX = 0;
static const uint32_t PAD_1_BASE_INDEX = 1000;

void GaInputMapAll(uint32_t deviceId, uint32_t baseIndex)
{
    g_GaInputMap->MapFloat(baseIndex + gainput::PadButtonLeftStickX, deviceId, gainput::PadButtonLeftStickX);
    g_GaInputMap->MapFloat(baseIndex + gainput::PadButtonLeftStickY, deviceId, gainput::PadButtonLeftStickY);
    g_GaInputMap->MapFloat(baseIndex + gainput::PadButtonRightStickX, deviceId, gainput::PadButtonRightStickX);
    g_GaInputMap->MapFloat(baseIndex + gainput::PadButtonRightStickY, deviceId, gainput::PadButtonRightStickY);
    g_GaInputMap->MapFloat(baseIndex + gainput::PadButtonAxis4, deviceId, gainput::PadButtonAxis4);
    g_GaInputMap->MapFloat(baseIndex + gainput::PadButtonAxis5, deviceId, gainput::PadButtonAxis5);

    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonLeft, deviceId, gainput::PadButtonLeft);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonRight, deviceId, gainput::PadButtonRight);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonUp, deviceId, gainput::PadButtonUp);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonDown, deviceId, gainput::PadButtonDown);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonA, deviceId, gainput::PadButtonA);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonB, deviceId, gainput::PadButtonB);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonX, deviceId, gainput::PadButtonX);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonY, deviceId, gainput::PadButtonY);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonL1, deviceId, gainput::PadButtonL1);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonR1, deviceId, gainput::PadButtonR1);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonL2, deviceId, gainput::PadButtonL2);
    g_GaInputMap->MapBool(baseIndex + gainput::PadButtonR2, deviceId, gainput::PadButtonR2);
}

InputManager::InputManager()
{
}

void InputManager::Init()
{
    assert(g_GaInputManager == nullptr);
    g_GaInputManager = new gainput::InputManager();
    g_GaInputMap = new gainput::InputMap(*g_GaInputManager);

    gainput::DeviceId devId1 = g_GaInputManager->CreateDevice<gainput::InputDevicePad>();
    gainput::DeviceId devId2 = g_GaInputManager->CreateDevice<gainput::InputDevicePad>();
    gainput::InputMap inputMap(*g_GaInputManager);
    GaInputMapAll(devId1, PAD_0_BASE_INDEX);
    GaInputMapAll(devId2, PAD_1_BASE_INDEX);
}

InputManager::~InputManager()
{
    delete g_GaInputMap; g_GaInputMap = nullptr;
    delete g_GaInputManager; g_GaInputManager = nullptr;
}

void InputManager::Clear()
{
    m_Controllers.clear();
}

void InputManager::Update()
{
    g_GaInputManager->Update();

    ControllerState state;

    state.device = InputDevice::Pad0;
    state.leftStick.x = g_GaInputMap->GetFloat(PAD_0_BASE_INDEX + gainput::PadButtonLeftStickX);
    state.leftStick.y = g_GaInputMap->GetFloat(PAD_0_BASE_INDEX + gainput::PadButtonLeftStickY);
    state.rightStick.x = g_GaInputMap->GetFloat(PAD_0_BASE_INDEX + gainput::PadButtonRightStickX);
    state.rightStick.y = g_GaInputMap->GetFloat(PAD_0_BASE_INDEX + gainput::PadButtonRightStickY);
    state.fire = g_GaInputMap->GetBool(PAD_0_BASE_INDEX + gainput::PadButtonA) ||
        g_GaInputMap->GetBool(PAD_0_BASE_INDEX + gainput::PadButtonB) ||
        g_GaInputMap->GetBool(PAD_0_BASE_INDEX + gainput::PadButtonX) ||
        g_GaInputMap->GetBool(PAD_0_BASE_INDEX + gainput::PadButtonY) ||
        g_GaInputMap->GetBool(PAD_0_BASE_INDEX + gainput::PadButtonL1) ||
        g_GaInputMap->GetBool(PAD_0_BASE_INDEX + gainput::PadButtonL2) ||
        g_GaInputMap->GetBool(PAD_0_BASE_INDEX + gainput::PadButtonR1) ||
        g_GaInputMap->GetBool(PAD_0_BASE_INDEX + gainput::PadButtonR2);
    PutState(state);

    state.device = InputDevice::Pad1;
    state.leftStick.x = g_GaInputMap->GetFloat(PAD_1_BASE_INDEX + gainput::PadButtonLeftStickX);
    state.leftStick.y = g_GaInputMap->GetFloat(PAD_1_BASE_INDEX + gainput::PadButtonLeftStickY);
    state.rightStick.x = g_GaInputMap->GetFloat(PAD_1_BASE_INDEX + gainput::PadButtonRightStickX);
    state.rightStick.y = g_GaInputMap->GetFloat(PAD_1_BASE_INDEX + gainput::PadButtonRightStickY);
    state.fire = g_GaInputMap->GetBool(PAD_1_BASE_INDEX + gainput::PadButtonA) ||
        g_GaInputMap->GetBool(PAD_1_BASE_INDEX + gainput::PadButtonX) ||
        g_GaInputMap->GetBool(PAD_1_BASE_INDEX + gainput::PadButtonY) ||
        g_GaInputMap->GetBool(PAD_1_BASE_INDEX + gainput::PadButtonL1) ||
        g_GaInputMap->GetBool(PAD_1_BASE_INDEX + gainput::PadButtonL2) ||
        g_GaInputMap->GetBool(PAD_1_BASE_INDEX + gainput::PadButtonR1) ||
        g_GaInputMap->GetBool(PAD_1_BASE_INDEX + gainput::PadButtonR2);
    PutState(state);

    state.device = InputDevice::KeyboardAndMouse;
    state.leftStick.x = 0.f;
    if(Dev.GetKeyState('A')) state.leftStick.x -= 1.f;
    if(Dev.GetKeyState('D')) state.leftStick.x += 1.f;
    state.leftStick.y = 0.f;
    if(Dev.GetKeyState('S')) state.leftStick.y -= 1.f;
    if(Dev.GetKeyState('W')) state.leftStick.y += 1.f;
    int mouseDx, mouseDy;
    Dev.GetMouseDelta(mouseDx, mouseDy);
    if(mouseDx || mouseDy)
    {
		const float sens = 0.01f;
        //m_LastMouseDelta *= 0.5;
        m_LastMouseDelta.x += (float)mouseDx * sens;
        m_LastMouseDelta.y += (float)-mouseDy * sens;
        //m_LastMouseDelta.x = max(-1.f, min(m_LastMouseDelta.x, 1.f));
        //m_LastMouseDelta.y = max(-1.f, min(m_LastMouseDelta.y, 1.f));
		if( m_LastMouseDelta.length2() > 1.f )
			m_LastMouseDelta.normalize();
    }
    state.rightStick = m_LastMouseDelta;
    state.fire = Dev.GetKeyState(VK_LBUTTON);
    PutState(state);
}

void InputManager::PutState(const ControllerState& state)
{
    for(size_t i = 0; i < m_Controllers.size(); ++i)
    {
        if(m_Controllers[i].device == state.device)
        {
            m_Controllers[i] = state;
            return;
        }
    }

    if(state.fire)
    {
        m_Controllers.push_back(state);
    }
}
