#pragma once

class InputManager
{
public:
    enum class InputDevice
    {
        Pad0, Pad1, KeyboardAndMouse,
    };

    struct ControllerState
    {
        InputDevice device;
        vec2 leftStick;
        vec2 rightStick;
        bool fire;
    };

    std::vector<ControllerState> m_Controllers;

    InputManager();
    void Init();
    ~InputManager();

    size_t GetPlayerCount() const { return m_Controllers.size(); }
	float  GetPlayerLeftX(int id) const { return (id>=0 && id<(int)m_Controllers.size()) ? m_Controllers[id].leftStick.x : 0; }
	float  GetPlayerLeftY(int id) const { return (id>=0 && id<(int)m_Controllers.size()) ? m_Controllers[id].leftStick.y : 0; }
	float  GetPlayerRightX(int id) const { return (id>=0 && id<(int)m_Controllers.size()) ? m_Controllers[id].rightStick.x : 0; }
	float  GetPlayerRightY(int id) const { return (id>=0 && id<(int)m_Controllers.size()) ? m_Controllers[id].rightStick.y : 0; }
	bool   GetPlayerFire(int id) const { return (id>=0 && id<(int)m_Controllers.size()) ? m_Controllers[id].fire : 0; }
	bool   GetPlayerIsMouse(int id) const { return (id>=0 && id<(int)m_Controllers.size()) ? m_Controllers[id].device==InputDevice::KeyboardAndMouse : 0; }


    void Clear();
    void Update();

private:
    vec2 m_LastMouseDelta = vec2(0.f, 0.f);

    void PutState(const ControllerState& state);
};

extern InputManager* g_InputManager;
