#pragma once

class TexturePackerDocument
{
public:
    struct Rect { int x, y, w, h; };
    struct Size { int w, h; };

    struct Frame
    {
        std::string fileName;
        Rect frame;
    };

    Size m_Size = {0, 0};
    float m_SizeX_inv = 0.f;
    float m_SizeY_inv = 0.f;
    std::vector<Frame> m_Frames;

    TexturePackerDocument();
    ~TexturePackerDocument();
    void Load(const char* filePath);

    // Case-insensitive comparison.
    // If not found, returns SIZE_MAX.
    size_t FrameFileNameToIndex(const char* fileName) const;

    void GetTexCoords(float* outTexCoords, size_t frameIndex) const;

private:
    void LoadMeta(const rapidjson::Value& val);
    void LoadFrame(const char* fileName, const rapidjson::Value& val);
    void LoadParam_Bool(bool& out, const rapidjson::Value& objVal, const char* paramName);
    void LoadParam_Int(int& out, const rapidjson::Value& objVal, const char* paramName);
    void LoadParam_Rect(Rect& out, const rapidjson::Value& objVal, const char* paramName);
    void LoadParam_Size(Size& out, const rapidjson::Value& objVal, const char* paramName);
};

extern DevTexture g_TexturePackerTexture;

class Animation
{
public:
    void Set(
        const char* const* frameNames,
        size_t frameCount,
        float singleFrameTime);

    size_t GetCurrentFrameIndex(float t) const;

private:
    float m_SingleFrameTime = 1.f;
    float m_TotalTime = 1.f;
    std::vector<size_t> m_FrameIndices;
};
