﻿
#pragma once



class EngineObject : public Collectable {
public:
	vec3		pos				= vec3(0, 0, 1.0f);
	float		render_scale	= 1;						// API	for 3d meshes		(must be after <pos: goes in obj_pos.w)
	vec3		front			= vec3(0, 1, 0);
	vec2		size			= vec2(0.05f, 0.05f);
	vec3		vel				= vec3(0, 0, 0);
	int			render_mode		= 1;						// API	1: quad		2: mesh		3: crystal
	int			render_layers	= 1;						// API
	float		render_thickness= 0;						// API
	int			render_attach	= 0;						// API	crystal number 0..4
	int			physics_mode	= 0;						// API	1: enable collisions		2: detach on collision
	float		physics_gravity = 0.0f;						// API	gravity factor
	float		physics_radius	= 0.05f;					// API	collision radius
	float		physics_sink	= 0.05f;					// API	offset for heightmap collisions (sink object to make it lie flat on the surface)
	int			wall_collision	= 0;						// API	engine sets it during collision with heightmap		(reset manually)
	float		update_time		= 0;						// API	time before update
	float		level_min		= 0.7f;						// API	stick <pos> to this radius range, keep <front> flat		(set to zero to disable)
	float		level_max		= 1.1f;						// API
	int			hit_faction		= 0;						// API	faction mask
	int			hit_colmask		= 0;						// API	bitmask of target collider factions						(calls on on_object_hit)
	float       energy          = 1.f;                      // API  0..1
	EngineMesh	*render_mesh = NULL;


	// API functions

	void SetPosition(float x, float y, float z);
	void SetFront(float x, float y, float z);				// immediately normalizes it
	void SetVelocity(float x, float y, float z);			// physics_gravity>0	-->		changes only perpendicular component of velocity (keep gravity)
	void SetVelocityForce(float x, float y, float z);		// forced - direct update velocity
	void SetSize(float x, float y);
	void SetStaticFrame(const char* name);
    void SetAnimation(const char* name);
	void SetMesh(const char *name);

	float GetPositionX() { return pos.x; }
	float GetPositionY() { return pos.y; }
	float GetPositionZ() { return pos.z; }
	float GetFrontX() { return front.x; }
	float GetFrontY() { return front.y; }
	float GetFrontZ() { return front.z; }
	float GetVelocityX() { return vel.x; }
	float GetVelocityY() { return vel.y; }
	float GetVelocityZ() { return vel.z; }


	// Internal functions
	void TickPhysics(float delta);
    size_t GetCurrentFrameIndex();

private:
    size_t m_StaticFrameIndex = SIZE_MAX;
    const Animation* m_Animation = nullptr;
    float m_AnimStartTime = 0.f;
};
