#include "pch.h"
#include "common.h"
#include "FmodSound.hpp"
#include "InputManager.h"
#include "ui.h"

static const char* const APP_NAME = "lazerbugz - Global Game Jam 2019";

extern DevTexture t_random;
extern int g_skiplevel;
extern int g_maxlevel;

GameUI ui;

DWORD fn_random(float u,float v)
{
	float a = (rand()&0xFF)*(3.141593f*2/256);
	return vec3(cos(a)*0.5f+0.5f,sin(a)*0.5f+0.5f,0).make_rgb();
}

DevTexture g_LoadingScreenTexture;
DevTexture g_SplashScreenTexture;

static void LoadingScreen()
{
    g_LoadingScreenTexture.Load("data/LoadingScreen.png");
    const vec2 textureSize = g_LoadingScreenTexture.GetSize2D();

    Dev.MainLoop();

    {
        Dev->Clear(0,NULL,D3DCLEAR_TARGET,0xFF404040,1.f,0);

        Dev->SetTexture(0, g_LoadingScreenTexture.GetTexture());

        Dev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
        Dev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
        Dev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_TFACTOR);
        Dev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
        Dev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
        Dev->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_TFACTOR);

        Dev->SetRenderState(D3DRS_TEXTUREFACTOR, 0xFFFF8080);

        Dev->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
        Dev->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
        Dev->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
        Dev->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
        Dev->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);

        const vec2 screenSize = Dev.GetScreenSizeV();
        const float MARGIN = 16.f;

        Dev.DrawScreenQuad(
            screenSize.x - MARGIN - textureSize.x,
            screenSize.y - MARGIN - textureSize.y,
            screenSize.x - MARGIN,
            screenSize.y - MARGIN);
    }

    Dev.MainLoop();

    g_LoadingScreenTexture.Unload();
}

static void SplashScreen()
{
    const float FADE_DURATION = 1.f;
    enum State {
        State_FadeIn,
        State_Showing,
        State_FadeOut,
        State_End
    } state = State_FadeIn;
    float stateStartTime = Dev.GetElapsedTime();

    g_SplashScreenTexture.Load("data/splash_pixel.png");
    const vec2 textureSize = g_SplashScreenTexture.GetSize2D();

    while(Dev.MainLoop() && state != State_End)
    {
        if(state == State_Showing)
        {
            if(Dev.GetKeyState(VK_SPACE) || Dev.GetKeyState(VK_RETURN) ||
                Dev.GetKeyState(VK_CONTROL) || Dev.GetKeyState(VK_ESCAPE) ||
                Dev.GetKeyState(VK_LBUTTON) || Dev.GetKeyState(VK_RBUTTON) || Dev.GetKeyState(VK_MBUTTON))
            {
                state = State_FadeOut;
                stateStartTime = Dev.GetElapsedTime();
            }
        }

        Dev->Clear(0,NULL,D3DCLEAR_TARGET,0xFFeceded,1.f,0);

        float alpha_f = 1.f;
        if(state != State_Showing)
        {
            const float progress = (Dev.GetElapsedTime() - stateStartTime) / FADE_DURATION;
            if(progress >= 1.f)
            {
                state = (State)(state + 1);
                stateStartTime = Dev.GetElapsedTime();
            }
 
            switch(state)
            {
            case State_FadeIn:
                alpha_f = max(0.f, min(progress, 1.f));
                break;
            case State_FadeOut:
                alpha_f = 1.f - max(0.f, min(progress, 1.f));
                break;
            case State_End:
                alpha_f = 0.f;
                break;
            }
        }

        const uint32_t alpha = (uint32_t)(alpha_f * 255.f + 0.5f);
        if(alpha)
        {
            const uint32_t color = 0x00FFFFFF | ((uint32_t)alpha << 24);
            Dev->SetRenderState(D3DRS_TEXTUREFACTOR, color);

            Dev->SetTexture(0, g_SplashScreenTexture.GetTexture());

            Dev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
            Dev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
            Dev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_TFACTOR);
            Dev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
            Dev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
            Dev->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_TFACTOR);

            Dev->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
            Dev->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
            Dev->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
            Dev->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
            Dev->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);

            Dev->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
            Dev->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
            Dev->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
            Dev->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

            const vec2 screenSize = Dev.GetScreenSizeV();
            const float MARGIN = 16.f;

            const float halfSizeX = textureSize.x / textureSize.y * screenSize.y * 0.5f;

            Dev.DrawScreenQuad(
                screenSize.x * 0.5f - halfSizeX,
                0.f,
                screenSize.x * 0.5f + halfSizeX,
                screenSize.y);
        }
    }

    g_SplashScreenTexture.Unload();
}

//Sound::Sound g_TestSnd("c:/Windows/media/tada.wav");

int WINAPI WinMain(HINSTANCE hInst,HINSTANCE hPrevInst,LPSTR pCmdLine,int nShowCmd)
{
    const bool splashScreenEnabled = GetConfigInt("General", "SplashScreenEnabled", "1") != 0;
    int sx = GetConfigInt("Graphics", "ResolutionX", "1024");
    int sy = GetConfigInt("Graphics", "ResolutionY", "768");
    // Doesn't work anyway.
	//if(pCmdLine)
	//	sscanf(pCmdLine,"%d%d",&sx,&sy);
	Dev.SetResolution(sx,sy,false);
	Dev.SetMouseCapture(true);
	Dev.Init();
    Dev.SetAppName(APP_NAME);

    LoadingScreen();

    g_InputManager = new InputManager();
    g_InputManager->Init();

    bool ok = g_TexturePackerTexture.Load(TEXTURE_ATLAS_TEXTURE_FILE_PATH);
    assert(ok);

    Sound::System soundSystem;
    //soundSystem.StartMusic("c:/Pub/Nitrous Oxide - Amnesia.mp3", true, 0.1f);

    t_random.BuildLookup2D(256,256,fn_random,false);

	sq.Init();
	SqRegister();

    game.Init();

    g_GameUi = new GameUI2();
    g_GameUi->Init();

    ui.Init();
//	ui.PopMessage(welcome1,500);
//	ui.PopMessage(welcome2,500);
	game.editor_mode = true;
	game.InitLevel(4,INIT_ALL|INIT_RESET_COMPLETED);

	sq.AddFile("scripts/main.nut");

    if(splashScreenEnabled)
    {
        SplashScreen();
    }

	int mx, my;
	Dev.GetMouseDelta(mx,my);

	while(Dev.MainLoop())
	{
		sq.CheckFiles(SqVM::CHECK_RUN_ALL);

        soundSystem.Update();
		if( !game.editor_mode )
			g_InputManager->Update();
        g_GameUi->Update();

        // Input test
        //printf("Pad: %g\n", g_InputMap->GetFloat(PAD_0_BASE_INDEX + gainput::PadButtonLeftStickX);
        
        // Sound test
        //if(Dev.GetKeyStroke('A')) g_TestSnd.Play();

        // Run program test
        /*
        if(Dev.GetKeyStroke('X'))
        {
            game.RunProgram("iexplore", "http://wp.pl/");
        }
        */

        Dev->Clear(0,NULL,D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER|D3DCLEAR_STENCIL,0x000000,1.f,0);

		float d = Dev.GetTimeDelta();
		ui.Tick(d);
		if( !ui.IsActive() )
		{
			game.Run(d);
		}

		if( g_skiplevel )
		{
			int level = game.level_id+g_skiplevel;
			if( level<0 ) level = 0;
			if( level>g_maxlevel ) level = g_maxlevel;
			if( level!=game.level_id )
			{
				ui.ClearMessages();
				game.InitLevel(level, INIT_ALL);
			}
			g_skiplevel = 0;
		}

        // TEST
        /*
        if(g_InputManager->GetPlayerCount())
        {
            printf("%g,%g %g,%g %i\n",
                g_InputManager->m_Controllers[0].leftStick.x,
                g_InputManager->m_Controllers[0].leftStick.y,
                g_InputManager->m_Controllers[0].rightStick.x,
                g_InputManager->m_Controllers[0].rightStick.y,
                g_InputManager->m_Controllers[0].fire);
        }
        */

        if(ui.IsShowGame())
        {
            game.RenderAll();
        }
        ui.Render();
        g_GameUi->Render();
    }

    delete g_GameUi; g_GameUi = nullptr;
    delete g_InputManager; g_InputManager = nullptr;
    
    return 0;
}


int main()
{
	return WinMain((HINSTANCE)GetModuleHandle(NULL), 0, nullptr, 0);
}
