#include "pch.h"
#include "common.h"



void Material::Load(const char *name)
{
	mname = name;

	string fname;
	const char *s = name;
	while( *s && *s!='/' )
		s++;
	fname.assign(name, s);

	if( *s=='/' )
		s++;

	m_shininess = 16.f;
	m_speccolor = vec3(1, 1, 1);
	m_mistcolor = vec3(1, 0.25f, 0);
	m_exitcolor = vec3(0.02f, 0.125f, 1.f)*8;
	m_mist_min = 0.65f;
	m_mist_max = 0.78f;
	m_mist_density = 0.25f;
	m_heat = 1.5f;

	while( *s )
	{
		const char *p = s;
		while( *s && *s!=',' )
			s++;

		float f = 0;
		sscanf(p+1, "%f", &f);

		switch( *p )
		{
		case 's': m_shininess = f;		break;
		case 'r': m_speccolor.x = f;	break;
		case 'g': m_speccolor.y = f;	break;
		case 'b': m_speccolor.z = f;	break;
		case 'x': m_mistcolor.x = f;	break;
		case 'y': m_mistcolor.y = f;	break;
		case 'z': m_mistcolor.z = f;	break;
		case 'u': m_exitcolor.x = f;	break;
		case 'v': m_exitcolor.y = f;	break;
		case 'w': m_exitcolor.z = f;	break;
		case 'm': m_mist_max = f;		break;
		case 'n': m_mist_min = f;		break;
		case 'd': m_mist_density = f;	break;
		case 'h': m_heat = f;			break;
		}

		if( *s==',' )
			s++;
	}

	diffuse_specular.Load(format("textures/%s_d.jpg", fname.c_str()).c_str());
	normal_height.Load(format("textures/%s_n.jpg", fname.c_str()).c_str());
}

void Material::LoadNext(const vector<string> &mlist)
{
	if( mlist.size()==0 )
		return;

	for( int i=0; i<(int)mlist.size()-1; i++ )
		if( mname==mlist[i] )
		{
			Load(mlist[i+1].c_str());
			return;
		}
	Load(mlist[0].c_str());
}

void Material::Set(DevEffect &fx, const char *name)
{
	fx->SetTexture(format("t_diffspec_%s", name).c_str(), diffuse_specular.GetTexture());
	fx->SetTexture(format("t_normheight_%s", name).c_str(), normal_height.GetTexture());

	float spec[4] ={
		m_speccolor.x,
		m_speccolor.y,
		m_speccolor.z,
		m_shininess
	};
	fx->SetFloatArray(format("m_specular_%s", name).c_str(), spec, 4);
}
