#include "pch.h"
#include "common.h"


DevRenderTarget postprocess_rt0(D3DFMT_A8R8G8B8,64,64,0,RT_PRIORITY(5));
DevRenderTarget postprocess_rt1(D3DFMT_A8R8G8B8,64,64,0,RT_PRIORITY(5));
DevRenderTarget postprocess_rt2(D3DFMT_A8R8G8B8,64,64,0,RT_PRIORITY(5));

DevEffect fx_postprocess("postprocess.fx");



void BuildSeparableKernel(vector<vec3> &kernel,float (*kernel_fn)(float),float size)
{
	vector<float> k_weights;
	vector<float> weights;

	// gather weights
	while(true)
	{
		float p = float(k_weights.size()) / size;
		if(p>=1) break;
		//k_weights.push_back(kernel_fn(p));
//		k_weights.push_back( cos(p*M_PI)*0.5f + 0.5f );
		k_weights.push_back( exp(-2*p*p) );
	}

	// make mirrored
	for(int i=k_weights.size()-1;i>0;i--)
		weights.push_back(k_weights[i]);
	for(int i=0;i<(int)k_weights.size();i++)
		weights.push_back(k_weights[i]);
	weights.push_back(0);

	// normalize
	float wsum = 0;
	for(int i=0;i<(int)weights.size();i++)
		wsum += weights[i];
	for(int i=0;i<(int)weights.size();i++)
		weights[i] /= wsum;

	// horizontal pass
	kernel.clear();
	for(int i=0;i<(int)weights.size();i+=2)
	{
		vec3 v;
		v.z = weights[i] + weights[i+1];
		v.x = i - int(k_weights.size()) + 1 + weights[i+1]/(weights[i] + weights[i+1]);
		v.y = 0;
		kernel.push_back(v);
	}
	kernel.push_back(vec3(0,0,0));

	// vertical pass
	for(int i=0;kernel[i].z>0;i++)
	{
		vec3 v = kernel[i];
		v.y = v.x;
		v.x = 0;
		kernel.push_back(v);
	}
	kernel.push_back(vec3(0,0,0));
	kernel.push_back(vec3(0,0,0));
}

void Convolve(const char *cmd,const vec3 *kernel,int n_kernel,int rt_w,int rt_h)
{
	static vec2 texcoord_offsets[8];
	static vec2 sample_offsets[24];
	static vec4 sample_weights[8];
	memset(texcoord_offsets,0,sizeof(texcoord_offsets));
	memset(sample_offsets,0,sizeof(sample_offsets));
	memset(sample_weights,0,sizeof(sample_weights));

	assert(n_kernel<=32);

	for(int i=0;i<n_kernel;i++)
	{
		vec2 offs;
		offs.x = kernel[i].x / rt_w;
		offs.y = kernel[i].y / rt_h;

		if(i<8)	texcoord_offsets[i] = offs;
		else	sample_offsets[i-8] = offs - texcoord_offsets[0];

		((float*)&sample_weights)[i] = kernel[i].z;
	}


	struct Vertex {
		float x, y, z;
		float u, v;
	};

	Vertex vtx[4];
	for(int i=0;i<4;i++)
	{
		vtx[i].x = ((i==0 || i==3) ? -1 :  1) - 1.0f/rt_w;
		vtx[i].y = ((i==0 || i==1) ?  1 : -1) + 1.0f/rt_h;
		vtx[i].z = 0;
		vtx[i].u = ((i==0 || i==3) ? 0.f : 1.f);
		vtx[i].v = ((i==0 || i==1) ? 0.f : 1.f);
	}


	fx_postprocess.StartTechnique(format("%s%d",cmd,n_kernel).c_str());
	fx_postprocess->SetFloatArray("texcoord_offsets",(float*)&texcoord_offsets,sizeof(texcoord_offsets)/sizeof(float));
	fx_postprocess->SetFloatArray("sample_offsets",(float*)&sample_offsets,sizeof(sample_offsets)/sizeof(float));
	fx_postprocess->SetFloatArray("sample_weights",(float*)&sample_weights,sizeof(sample_weights)/sizeof(float));

	Dev->SetFVF(D3DFVF_XYZ|D3DFVF_TEX1);
	while(fx_postprocess.StartPass())
		Dev->DrawPrimitiveUP(D3DPT_TRIANGLEFAN,2,vtx,sizeof(Vertex));
}

void PostProcess()
{
	bool rt_flip = false;
	int rt_w, rt_h;

	D3DVIEWPORT9 camera_viewport;
	RECT camera_viewport_rect;
	Dev->GetViewport(&camera_viewport);
	camera_viewport_rect.left = camera_viewport.X;
	camera_viewport_rect.top = camera_viewport.Y;
	camera_viewport_rect.right = camera_viewport.X + camera_viewport.Width;
	camera_viewport_rect.bottom = camera_viewport.Y + camera_viewport.Height;

	// setup render targets
	postprocess_rt0.SetParameters(D3DFMT_A8R8G8B8,camera_viewport.Width/2,camera_viewport.Height/2,0);
	postprocess_rt1.SetParameters(D3DFMT_A8R8G8B8,camera_viewport.Width/4,camera_viewport.Height/4,0);
	postprocess_rt2.SetParameters(D3DFMT_A8R8G8B8,camera_viewport.Width/4,camera_viewport.Height/4,0);

	postprocess_rt1.GetCurrentSize(rt_w,rt_h);

	// copy scene and stretch down
	Dev.StretchRect(NULL,&camera_viewport_rect,&postprocess_rt0,NULL,D3DTEXF_LINEAR);

	// stretch down and bright pass
	static const vec3 QUAD[] = {
		vec3(-0.25f,-0.25f,0.25f),
		vec3(-0.25f, 0.25f,0.25f),
		vec3( 0.25f,-0.25f,0.25f),
		vec3( 0.25f, 0.25f,0.25f)
	};

	Dev.SetRenderTarget(0,&postprocess_rt1);
	fx_postprocess->SetTexture("tex",postprocess_rt0.GetTexture());
	Convolve("BrightPass",QUAD,4,rt_w,rt_h);

	// build kernel
	static vector<vec3> kernel_data;
	float size = rt_h / 10.f;
	if(size>32) size = 32;
	BuildSeparableKernel(kernel_data,NULL,size);

	vec3 *kernels = &kernel_data[0];
	while(kernels->z > 0)
	{
		vec3 *end = kernels;
		while(end->z >0) end++;

		fx_postprocess->SetTexture("tex",(rt_flip ? &postprocess_rt2 : &postprocess_rt1)->GetTexture());
		Dev.SetRenderTarget(0,(rt_flip ? &postprocess_rt1 : &postprocess_rt2));

		Convolve("Sampler",kernels,end - kernels,rt_w,rt_h);
		kernels = end;
		kernels++;

		rt_flip = !rt_flip;
	}

	fx_postprocess->SetTexture("tex",(rt_flip ? &postprocess_rt2 : &postprocess_rt1)->GetTexture());
	Dev.SetRenderTarget(0,NULL);
	Dev->SetViewport(&camera_viewport);

	fx_postprocess.StartTechnique("Compose");
	while(fx_postprocess.StartPass())
		Dev.DrawScreenQuad(
				(float)camera_viewport_rect.left,
				(float)camera_viewport_rect.top,
				(float)camera_viewport_rect.right,
				(float)camera_viewport_rect.bottom );
}
