
float2 texcoord_offsets[8];
float2 sample_offsets[24];
float4 sample_weights[8];
float4 ps_color;


texture tex;

sampler s_tex_p = sampler_state {
	Texture = <tex>;
	MinFilter = Point;
	MagFilter = Point;
	MipFilter = None;
	AddressU = Clamp;
	AddressV = Clamp;
};

sampler s_tex_f = sampler_state {
	Texture = <tex>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
	AddressU = Clamp;
	AddressV = Clamp;
};



void VS_Sampler(
		float4 pos			: POSITION,
		float2 tc			: TEXCOORD0,
		out float4 hpos		: POSITION,
		out float2 tc0		: TEXCOORD0,
		out float2 tc1		: TEXCOORD1,
		out float2 tc2		: TEXCOORD2,
		out float2 tc3		: TEXCOORD3,
		out float2 tc4		: TEXCOORD4,
		out float2 tc5		: TEXCOORD5,
		out float2 tc6		: TEXCOORD6,
		out float2 tc7		: TEXCOORD7 )
{
	hpos = pos;
	tc0 = tc + texcoord_offsets[0];
	tc1 = tc + texcoord_offsets[1];
	tc2 = tc + texcoord_offsets[2];
	tc3 = tc + texcoord_offsets[3];
	tc4 = tc + texcoord_offsets[4];
	tc5 = tc + texcoord_offsets[5];
	tc6 = tc + texcoord_offsets[6];
	tc7 = tc + texcoord_offsets[7];
}

float4 PS_Sampler(
			uniform sampler s_tex,
			uniform int cnt,
			float2 tc0		: TEXCOORD0,
			float2 tc1		: TEXCOORD1,
			float2 tc2		: TEXCOORD2,
			float2 tc3		: TEXCOORD3,
			float2 tc4		: TEXCOORD4,
			float2 tc5		: TEXCOORD5,
			float2 tc6		: TEXCOORD6,
			float2 tc7		: TEXCOORD7 ) : COLOR0
{
	float4 s1;
	if(cnt>= 1) s1  = tex2D(s_tex,tc0) * sample_weights[0].x;
	if(cnt>= 2) s1 += tex2D(s_tex,tc1) * sample_weights[0].y;
	if(cnt>= 3) s1 += tex2D(s_tex,tc2) * sample_weights[0].z;
	if(cnt>= 4) s1 += tex2D(s_tex,tc3) * sample_weights[0].w;
	if(cnt>= 5) s1 += tex2D(s_tex,tc4) * sample_weights[1].x;
	if(cnt>= 6) s1 += tex2D(s_tex,tc5) * sample_weights[1].y;
	if(cnt>= 7) s1 += tex2D(s_tex,tc6) * sample_weights[1].z;
	if(cnt>= 8) s1 += tex2D(s_tex,tc7) * sample_weights[1].w;
	if(cnt>= 9) s1 += tex2D(s_tex,tc0 + sample_offsets[0]) * sample_weights[2].x;
	if(cnt>=10) s1 += tex2D(s_tex,tc0 + sample_offsets[1]) * sample_weights[2].y;
	if(cnt>=11) s1 += tex2D(s_tex,tc0 + sample_offsets[2]) * sample_weights[2].z;
	if(cnt>=12) s1 += tex2D(s_tex,tc0 + sample_offsets[3]) * sample_weights[2].w;
	if(cnt>=13) s1 += tex2D(s_tex,tc0 + sample_offsets[4]) * sample_weights[3].x;
	if(cnt>=14) s1 += tex2D(s_tex,tc0 + sample_offsets[5]) * sample_weights[3].y;
	if(cnt>=15) s1 += tex2D(s_tex,tc0 + sample_offsets[6]) * sample_weights[3].z;
	if(cnt>=16) s1 += tex2D(s_tex,tc0 + sample_offsets[7]) * sample_weights[3].w;
	if(cnt>=17) s1 += tex2D(s_tex,tc0 + sample_offsets[8]) * sample_weights[4].x;
	if(cnt>=18) s1 += tex2D(s_tex,tc0 + sample_offsets[9]) * sample_weights[4].y;
	if(cnt>=19) s1 += tex2D(s_tex,tc0 + sample_offsets[10]) * sample_weights[4].z;
	if(cnt>=20) s1 += tex2D(s_tex,tc0 + sample_offsets[11]) * sample_weights[4].w;
	if(cnt>=21) s1 += tex2D(s_tex,tc0 + sample_offsets[12]) * sample_weights[5].x;
	if(cnt>=22) s1 += tex2D(s_tex,tc0 + sample_offsets[13]) * sample_weights[5].y;
	if(cnt>=23) s1 += tex2D(s_tex,tc0 + sample_offsets[14]) * sample_weights[5].z;
	if(cnt>=24) s1 += tex2D(s_tex,tc0 + sample_offsets[15]) * sample_weights[5].w;
	if(cnt>=25) s1 += tex2D(s_tex,tc0 + sample_offsets[16]) * sample_weights[6].x;
	if(cnt>=26) s1 += tex2D(s_tex,tc0 + sample_offsets[17]) * sample_weights[6].y;
	if(cnt>=27) s1 += tex2D(s_tex,tc0 + sample_offsets[18]) * sample_weights[6].z;
	if(cnt>=28) s1 += tex2D(s_tex,tc0 + sample_offsets[19]) * sample_weights[6].w;
	if(cnt>=29) s1 += tex2D(s_tex,tc0 + sample_offsets[20]) * sample_weights[7].x;
	if(cnt>=30) s1 += tex2D(s_tex,tc0 + sample_offsets[21]) * sample_weights[7].y;
	if(cnt>=31) s1 += tex2D(s_tex,tc0 + sample_offsets[22]) * sample_weights[7].z;
	if(cnt>=32) s1 += tex2D(s_tex,tc0 + sample_offsets[23]) * sample_weights[7].w;
	
	return s1;
}


float4 bright_pass(float4 c)
{
	float enl = saturate( (max(c.x,max(c.y,c.z))*2 - 1)*0.5 + c.w*2 );
	return c * enl;
}

float4 PS_BrightPass(
			uniform sampler s_tex,
			uniform int cnt,
			float2 tc0		: TEXCOORD0,
			float2 tc1		: TEXCOORD1,
			float2 tc2		: TEXCOORD2,
			float2 tc3		: TEXCOORD3 ) : COLOR0
{
	float4 s1;
	s1  = bright_pass( tex2D(s_tex,tc0) );
	s1 += bright_pass( tex2D(s_tex,tc1) );
	s1 += bright_pass( tex2D(s_tex,tc2) );
	s1 += bright_pass( tex2D(s_tex,tc3) );
	return s1 * 0.25f;
}


float4 PS_Compose(float2 uv : TEXCOORD0) : COLOR0
{
	float4 c = tex2D(s_tex_f,uv);
	return c;
}

float4 PS_Color() : COLOR0
{
	return ps_color;
}

technique Compose {
	pass {
		PixelShader  = compile ps_2_0 PS_Compose();
		ZEnable = false;
		ZWriteEnable = false;
		AlphaBlendEnable = True;
		SrcBlend = InvDestColor;
		DestBlend = One;
	}
}

technique FadeWhite {
	pass {
		PixelShader = compile ps_2_0 PS_Color();
		ZEnable = false;
		ZWriteEnable = false;
		AlphaBlendEnable = True;
		SrcBlend = One;
		DestBlend = One;		
	}
}


#define TECH(cmd,smp,n)											\
	technique cmd ## n {										\
		pass {													\
			VertexShader = compile vs_2_0 VS_Sampler();			\
			PixelShader  = compile ps_2_0 PS_ ## cmd(smp,n);	\
			ZEnable = false;									\
			ZWriteEnable = false;								\
		}														\
	}															\

#define SMP(n)	TECH(Sampler,s_tex_f,n)

TECH(BrightPass,s_tex_p,4)
SMP( 1)
SMP( 2)
SMP( 3)
SMP( 4)
SMP( 5)
SMP( 6)
SMP( 7)
SMP( 8)
SMP( 9)
SMP(10)
SMP(11)
SMP(12)
SMP(13)
SMP(14)
SMP(15)
SMP(16)
SMP(17)
SMP(18)
SMP(19)
SMP(20)
SMP(21)
SMP(22)
SMP(23)
SMP(24)
SMP(25)
SMP(26)
SMP(27)
SMP(28)
SMP(29)
SMP(30)
SMP(31)
SMP(32)
