const SHOT_INTERVAL = 0.1;

class Player extends Object {

	player_number	= 0;
	in_left			= vec2(0,0);
	in_right		= vec2(0,1);
	in_fire			= 0;
	in_mouse_mode	= 0;
	cam_front		= vec3(0,0,0);
	cam_up			= vec3(0,0,0);
	cam_right		= vec3(0,0,0);
	last_shot_time  = 0;
	walking_counter = 0; // Positive when walking, negative when not.

	// "health" is now called "energy" and it's accessible implcitly.

	constructor(new_pos, pid) {
		base.constructor()

		player_number = pid;

		
		size = vec2(1,1)*0.05;
		physics_mode = 1;
		physics_gravity = GRAVITY;
		physics_radius = size.x;
		physics_sink = size.x*0.85
		pos = new_pos;
		front = vec3(1.0, 0.0, 0.0);
		level_max = 0.9;
		hit_faction = FACTION_PLAYER;

		push_data()
		SetStaticFrame("guy_0000.png");
		eng.AttachCamera(pid,this);
	}

	function read_input()
	{
		in_left = vec2( input.GetPlayerLeftX(player_number), input.GetPlayerLeftY(player_number) );
		local right = vec2( input.GetPlayerRightX(player_number), input.GetPlayerRightY(player_number) );
		if( length(right) > 0.1 )
			in_right = right;
		in_fire = input.GetPlayerFire(player_number) ? 1 : 0;
		in_mouse_mode = input.GetPlayerIsMouse(player_number) ? 1 : 0;

		cam_front = vec3(
			eng.GetCamFrontX(player_number),
			eng.GetCamFrontY(player_number),
			eng.GetCamFrontZ(player_number)
			);
		cam_up = vec3(
			eng.GetCamUpX(player_number),
			eng.GetCamUpY(player_number),
			eng.GetCamUpZ(player_number)
			);
		cam_right = normalize(cross(cam_up,cam_front));
	}

	function logic_update()
	{
		if (energy <= 0) {
			print("you died");
			print(alive_players);
			remove();
		}
		local t = eng.GetElapsedTime();
		read_input();
		alive_players++;

		local isWalking = abs(in_left.x) > 1e-3 || abs(in_left.y) > 1e-3;
		const WALKING_FRAMES_DELAY = 6;
		if(isWalking) {
			if(walking_counter >= 0) {
				++walking_counter;
			} else {
				walking_counter = 1;
			}
			if(walking_counter == WALKING_FRAMES_DELAY) {
				eng.StartSoundEffectLoop("FootSteps", 1.0);
				SetAnimation("GuyWalking");
			}
		} else {
			if(walking_counter <= 0) {
				--walking_counter;
			} else {
				walking_counter = -1;
			}
			if(walking_counter == -WALKING_FRAMES_DELAY) {
				eng.StopSoundEffectLoop("FootSteps");
				SetStaticFrame("guy_0000.png");
			}
		}

		if(in_fire && t >= last_shot_time + SHOT_INTERVAL)
		{
			shoot();
			last_shot_time = t;
		}


		local left3d = cam_right*in_left.x - cam_up*in_left.y;
		local right3d = cam_right*in_right.x - cam_up*in_right.y;

		if(length(right3d) > 1e-2) {
			local idvel = left3d;
			if(time_delta > 1e-3)
			{
				vel += (idvel-vel)*(1-pow(0.00001,time_delta));
			}
			front = right3d;
		}

		//regenetate health
		if( eng.FindClosestFaction(pos.x, pos.y, pos.z, 0.2, FACTION_HOMEBASE) )
		{
			if (energy < 1.0){
				energy += 0.2;
			}
		}

		// Handle pickups
		if( eng.FindClosestFaction(pos.x, pos.y, pos.z, physics_radius, FACTION_PICKUP) )
		{
			other.on_pickup(this);
		}

		
		update_time = 0;		// continuous update
	}

	function shoot()
	{
		//print("Pif paf!\n");
        eng.PlaySoundEffect("Laser", 1.0);
		Shot(pos + (front + right*0.06) * (length(size) * 1.01), front);
	}

}
