﻿
#include "common.h"
#include "InputManager.h"
using namespace Sqrat;


SqVM sq;




void xsq_print(const char *msg)
{
	printf("%s", msg);
}

void xsq_error(const char *err)
{
	printf("ERROR: %s", err);
}

bool xsq_getfile(const char *path, std::vector<unsigned char> &data)
{
	return NFS.GetFileBytes(path, data);
}

unsigned long long xsq_getfiletime(const char *path)
{
	return GetFileTime(path);
}

void script_add(const char *path)
{
	sq.AddFile(path);
}
XSQ_REGISTER_FN(script_add);

void print_vec3(vec3 v)
{
	printf("%f %f %f\n", v.x, v.y, v.z);
}
XSQ_REGISTER_FN(print_vec3);




void SqRegister()
{
	RootTable(sq()).Bind("Engine", Class<Engine, NoConstructor<Engine>>(sq(), "Engine")
        .Func("GetElapsedTime", &Engine::GetElapsedTime)
        .Func("GetTimeDelta", &Engine::GetTimeDelta)
		.Func("SetLocation", &Engine::SetLocation)
		.Func("UnlinkAllObjects", &Engine::UnlinkAllObjects)
		.Func("LinkObject", &Engine::LinkObject)
		.Func("UnlinkObject", &Engine::UnlinkObject)
		.Func("SetCameraCount", &Engine::SetCameraCount)
		.Func("AttachCamera", &Engine::AttachCamera)
		.Func("RemoveCamera", &Engine::RemoveCamera)
		.Func("GetCamFrontX", &Engine::GetCamFrontX)
		.Func("GetCamFrontY", &Engine::GetCamFrontY)
		.Func("GetCamFrontZ", &Engine::GetCamFrontZ)
		.Func("GetCamUpX", &Engine::GetCamUpX)
		.Func("GetCamUpY", &Engine::GetCamUpY)
        .Func("GetCamUpZ", &Engine::GetCamUpZ)
        .Func("PlaySoundEffect", &Engine::PlaySoundEffect)
        .Func("StartSoundEffectLoop", &Engine::StartSoundEffectLoop)
        .Func("StopSoundEffectLoop", &Engine::StopSoundEffectLoop)
        .Func("RunProgram", &Engine::RunProgram)
        .Func("GetKeyState", &Engine::GetKeyState)
        .Func("GetKeyStroke", &Engine::GetKeyStroke)
		.Func("FindClosestFaction", &Engine::FindClosestFaction)
	);

	RootTable(sq()).Bind("EngineObject", Class<EngineObject>(sq(), "EngineObject")
		.Var("render_mode", &EngineObject::render_mode)
		.Var("render_layers", &EngineObject::render_layers)
		.Var("render_thickness", &EngineObject::render_thickness)
		.Var("render_scale", &EngineObject::render_scale)
		.Var("render_attach", &EngineObject::render_attach)
		.Var("physics_mode", &EngineObject::physics_mode)
		.Var("physics_gravity", &EngineObject::physics_gravity)
		.Var("physics_radius", &EngineObject::physics_radius)
		.Var("physics_sink", &EngineObject::physics_sink)
		.Var("wall_collision", &EngineObject::wall_collision)
		.Var("update_time", &EngineObject::update_time)
		.Var("level_min", &EngineObject::level_min)
		.Var("level_max", &EngineObject::level_max)
		.Var("hit_faction", &EngineObject::hit_faction)
        .Var("hit_colmask", &EngineObject::hit_colmask)
        .Var("energy", &EngineObject::energy)
		.Func("SetPosition", &EngineObject::SetPosition)
		.Func("SetFront", &EngineObject::SetFront)
		.Func("SetVelocity", &EngineObject::SetVelocity)
		.Func("SetVelocityForce", &EngineObject::SetVelocityForce)
		.Func("SetSize", &EngineObject::SetSize)
        .Func("SetStaticFrame", &EngineObject::SetStaticFrame)
		.Func("SetAnimation", &EngineObject::SetAnimation)
		.Func("SetMesh", &EngineObject::SetMesh)
		.Func("GetPositionX", &EngineObject::GetPositionX)
		.Func("GetPositionY", &EngineObject::GetPositionY)
		.Func("GetPositionZ", &EngineObject::GetPositionZ)
		.Func("GetFrontX", &EngineObject::GetFrontX)
		.Func("GetFrontY", &EngineObject::GetFrontY)
		.Func("GetFrontZ", &EngineObject::GetFrontZ)
		.Func("GetVelocityX", &EngineObject::GetVelocityX)
		.Func("GetVelocityY", &EngineObject::GetVelocityY)
		.Func("GetVelocityZ", &EngineObject::GetVelocityZ)
	);

	RootTable(sq()).Bind("InputManager", Class<InputManager>(sq(), "InputManager")
		.Func("GetPlayerCount", &InputManager::GetPlayerCount)
		.Func("GetPlayerLeftX", &InputManager::GetPlayerLeftX)
		.Func("GetPlayerLeftY", &InputManager::GetPlayerLeftY)
		.Func("GetPlayerRightX", &InputManager::GetPlayerRightX)
		.Func("GetPlayerRightY", &InputManager::GetPlayerRightY)
		.Func("GetPlayerFire", &InputManager::GetPlayerFire)
		.Func("GetPlayerIsMouse", &InputManager::GetPlayerIsMouse)
		.Func("Clear", &InputManager::Clear)
	);

	RootTable(sq()).SetValue("eng", &game);
	RootTable(sq()).SetValue("input", g_InputManager);
}
