

#define R(p,q)	p=float2(p.x*cos(q)+p.y*sin(q),p.y*cos(q)-p.x*sin(q))

static const float near_plane = 0.5f;


float time;
float2 ssize;
float2 vppos;
float2 vpsize;
float vzoom;
float3 dir_front;
float3 dir_right;
float3 dir_up;
float3 ball1_pos;
float3 ball2_pos;
float3 ball3_pos;
float3 ball4_pos;
float3 ball5_pos;
float3 ball1_color;
float3 ball2_color;
float3 ball3_color;
float3 ball4_color;
float3 ball5_color;
float3 exit1_pos;
float3 exit2_pos;
float3 exit_color;
float3 exit_color2;
float3 exit_status;
float4 m_mist_params;
float4 m_mist_color;
float4 obj_pos;					// W: scale
float3 obj_front;


//#define M(v)	float3(dot(v,dir_right),dot(v,dir_up),dot(v,dir_front))
#define M(v)	(v.x*dir_right + v.y*dir_up + v.z*dir_front)



texture t_cube_c;
sampler s_cube_c = sampler_state {
	Texture = <t_cube_c>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
};

texture t_cube_h;
sampler s_cube_h = sampler_state {
	Texture = <t_cube_h>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
};

texture t_screen;
sampler s_screen = sampler_state {
	Texture = <t_screen>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
	AddressU = Clamp;
	AddressV = Clamp;
};

texture t_heat;
sampler s_heat = sampler_state {
	Texture = <t_heat>;
	MinFilter = Point;
	MagFilter = Point;
	MipFilter = None;
};

texture t_random;
sampler s_random = sampler_state {
	Texture = <t_random>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
};

texture t_object;
sampler s_object = sampler_state {
    Texture = <t_object>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

#define TEX(name)						\
	texture t_##name;					\
	sampler s_##name = sampler_state {	\
		Texture = <t_##name>;			\
		MinFilter = Linear;				\
		MagFilter = Linear;				\
		MipFilter = Point;				\
	};

TEX(diffspec_0);
TEX(normheight_0);
float4 m_specular_0;

TEX(diffspec_1);
TEX(normheight_1);
float4 m_specular_1;

TEX(diffspec_2);
TEX(normheight_2);
float4 m_specular_2;



float4 texPLANAR(sampler s,float3 p,float3 n)
{
	float4 tx = tex2D(s,p.yz);
	float4 ty = tex2D(s,p.zx);
	float4 tz = tex2D(s,p.xy);
	n = pow(abs(n),4);
	n /= dot(n,1);
	return tx*n.x + ty*n.y + tz*n.z;
}

float4 texPLANARnormal(sampler s,float3 p,float3 n)
{
	float3 ox = normalize(float3(1,0,0)-n*n.x);
	float3 oy = normalize(float3(0,1,0)-n*n.y);
	float3 oz = normalize(float3(0,0,1)-n*n.z);

	float4 tx = tex2D(s,p.yz);//.zxyw;
	float4 ty = tex2D(s,p.zx);//.yzxw;
	float4 tz = tex2D(s,p.xy);//.xyzw;
	tx.xyz = tx.xyz*2 - 1;
	ty.xyz = ty.xyz*2 - 1;
	tz.xyz = tz.xyz*2 - 1;
	tx.xyz = n *tx.z + oy*tx.x + oz*tx.y;
	ty.xyz = ox*ty.y + n *ty.z + oz*ty.x;
	tz.xyz = ox*tz.x + oy*tz.y + n *tz.z;
	
	n = pow(abs(n),4);
	n /= dot(n,1);
	return tx*n.x + ty*n.y + tz*n.z;
}


float2 sphere(float3 p,float3 d,float r)
{
	float A = dot(d,d);
	float B = 2*dot(p,d);
	float C = dot(p,p) - r*r;
	float D = B*B - 4*A*C;
	if(D<=0) return 0;
	return max( (-B+sqrt(D)*float2(-1,1))/(2*A), 0 );
}

float mystep(float a,float b,float x)
{
	return saturate(x/(b-a)-a/(b-a));
}



float trace_moon_shadow(float3 cpos,float3 rdir)
{
	float2 s1 = sphere(cpos,rdir,1.01);
	s1.x = max(s1.x,0);
	if(s1.x>=s1.y) return 1;

	float fp = s1.x;
	while(1)
	{
		fp += 0.025;
		if(fp>=s1.y)
			return 1;
		
		float3 p = cpos + rdir*fp;
		float hm = texCUBElod(s_cube_c,float4(p,0)).w;
		if(hm*hm - dot(p,p)>0) break;
	}
	
	return 0;
}


float2 trace_moon(float3 cpos,
				  float3 rdir,
				  float  dmax,
				  out float4 hm,
				  out float3 p )
{
	float2 s1 = sphere(cpos,rdir,1);
	if(s1.x<=0) return float2(0,1);

//	float2 s2 = sphere(cpos,rdir,0.65);
//	if(s2.x>0) s1.y = s2.x;

	s1.y = min(s1.y,dmax);

	float3 p1 = cpos + rdir*s1.x;
	float3 p2 = cpos + rdir*s1.y;
	
	float fp1 = 0, fp2 = 0;
	float minh=1;
	[unroll(32)] while(1)
	//for(int i=0;i<32;i++)
	{
		if(fp2>=1)
			return float2(0,minh);
		
		p = lerp(p1,p2,fp2);
		hm = texCUBElod(s_cube_c,float4(p,0));
		if(hm.w*hm.w - dot(p,p)>0) break;
		minh = min(minh,dot(p,p));

		fp1 = fp2;
		fp2 += 1./25;
		//fp2 += (10.0+i*i)/9000;
	}
	
	float fpm;
	for(int i=0;i<7;i++)
	{
		fpm = lerp(fp1,fp2,0.5);
		p = lerp(p1,p2,fpm);
		
		hm = texCUBElod(s_cube_c,float4(p,0));

		if(hm.w*hm.w - dot(p,p)>0)	fp2=fpm;
		else						fp1=fpm;
	}

	fpm = lerp(fp1,fp2,0.5);
	p = lerp(p1,p2,fpm);
	
	hm = texCUBElod(s_cube_c,float4(p,0));
	minh = min(minh,hm.w*hm.w);

	if(hm.w*hm.w - dot(p,p)>0)	fp2=fpm;
	else						fp1=fpm;
	
	return float2(s1.x,minh);
}

float4 shade_moon(float3 cpos,float3 rdir,float3 ldir,float4 hm,float3 p,float4 shd)
{
	float3 n = normalize(hm.xyz*2-1);
	
	float3 tc = p*1.5;
	float4 ds0=0,nh0=0,ds1=0,nh1=0,ds2=0,nh2=0;
	float h = hm.w;
	float q1 = smoothstep(0.85+0.005,0.85+0.10,h);
	float q2 = smoothstep(0.70+0.005,0.70+0.025,h);
	
	//float t1 = dot(normalize(p),exit1_pos);
	//float t2 = dot(normalize(p),exit2_pos);
	//t1 = pow(saturate(1-(1-t1)/0.03),0.5);
	//t2 = pow(saturate(1-(1-t2)/0.03),0.5);

//	if(q1>0)
	{
		ds0 = texPLANAR(s_diffspec_0,tc,n);
		nh0 = texPLANARnormal(s_normheight_0,tc,n);
	}
//	if(q1<1 && q2>0)
	{
		ds1 = texPLANAR(s_diffspec_1,tc,n);
		nh1 = texPLANARnormal(s_normheight_1,tc,n);
	}
//	if(q2<1)
	{
		ds2 = texPLANAR(s_diffspec_2,tc,n);
		nh2 = texPLANARnormal(s_normheight_2,tc,n);
		
		float t1=0, t2=0;
		float3 ec = exit_color*(1+max(t1*exit_status.x,max(t2*exit_status.y,14*exit_status.z)));
		float t = max(t1,max(t2,exit_status.z));
		ec *= dot(ds2.xyz,float3(0.30,0.59,0.11));
		ec = 1-pow(1.5,-ec);
		ds2.xyz = lerp(ds2.xyz,ec,t);
		nh2.w = lerp(nh2.w,1,t);
	}
	
	float4 mt0 = m_specular_0;
	float4 mt1 = m_specular_1;
	float4 mt2 = m_specular_2;
	
	float4 ds = lerp(ds2,lerp(ds1,ds0,q1),q2);
	float4 nh = lerp(nh2,lerp(nh1,nh0,q1),q2);
	float4 mt = lerp(mt2,lerp(mt1,mt0,q1),q2);
//	nh.w = max(nh.w,pow(ds.w*(1-ds.w)*4,4)*exit_status.z*0.1);
//	ds.xyz = lerp(ds.xyz,exit_color2,nh.w*exit_status.z);
	

	float4 e = float4(ds.xyz,1)*nh.w;
	ds.xyz -= e.xyz;
	nh.xyz = normalize(nh.xyz);
	
	float ss = saturate(dot(n,ldir)*10);
	float d = dot(nh.xyz,ldir);
	float s = pow(abs(dot(normalize(reflect(rdir,nh.xyz)),ldir)),mt.w);
	
	ds = (d*float4(ds.xyz,0) + s*float4(mt.xyz,1)*ds.w)*ss*shd + e*2;
//	ds += t*float4(0,0.3,1,1);
	ds.w *= ds.w*ds.w;
	
	return ds;
}


float2 trace_marble(float3 cpos,
					float3 rdir )
{
	return sphere(cpos,rdir,0.015);
}

float4 trace_marble_shadow(float3 cpos,
						   float3 rdir,
						   float3 color )
{
	float2 s = sphere(cpos,rdir,0.015);
	if(s.x<=0) return 1;
	float t = saturate((s.y-s.x)*(0.5/0.05));
	t = 0.02/(1.02-t);
	return saturate(t-s.x*4)*3*float4(color,0);
}

float4 shade_marble(float3 cpos,
					float3 rdir,
					float3 ldir,
					float2 sp,
					float3 color,
					float4 trans1,
					out float4 trans )
{
	float3 p = cpos+sp.x*rdir;
	float3 n = normalize(p);
	float3 p2 = cpos+sp.y*rdir;
	float3 n2 = -normalize(p2);
	float  t = saturate((sp.y-sp.x)*(0.5/0.05));
	
	float4 ds = float4(color,1);
	float4 mt = 32;
	

	float ss = dot(n,ldir)*0.5+0.5;
	float d = dot(n,ldir)*0.2;
	d += saturate(1-dot(-rdir,n));
	d += 0.02/(1.02-(1-t))*0.5;
	float3 r = normalize(reflect(rdir,n));
	float3 r2 = normalize(reflect(rdir,n2));
	float s = pow(abs(dot(r,ldir)),mt.x);
	d += pow(abs(dot(r2,ldir)),mt.x/2)*pow(0.2,t);
		
	ds = (d*float4(ds.xyz,0) + s)*ss;
	ds.w *= ds.w*ds.w;
	trans = trans1*float4(color,0)*pow(0.05,t);
	
	return ds;
}

//#define EXTENDED_SHADER_FEATURES
//#include "traceball_Raytrace.inc"
////#undef EXTENDED_SHADER_FEATURES
////#include "traceball_Raytrace.inc"

float4 PS_Raytrace_Full(float2 vpos : vpos, out float4 heat : color1, out float depth_out : DEPTH) : color0
{
	vpos = (vpos+.5-vppos-vpsize/2)/(vpsize.y*vzoom);
	heat = 0;
	depth_out = 1;
	
	float3 cpos = float3(0,0,-2);
	float3 rdir = normalize(float3(vpos,1));
	float3 ldir = normalize(float3(-1,-1,-1));
	cpos = M(cpos);
	rdir = M(rdir);
	ldir = M(ldir);
	
	float3 mrp1 = ball1_pos;
	float3 mrp2 = ball2_pos;
	float3 mrp3 = ball3_pos;
	float3 mrp4 = ball4_pos;
	float3 mrp5 = ball5_pos;
	float2 mr1 = trace_marble(cpos - mrp1, rdir);
	float2 mr2 = trace_marble(cpos - mrp2, rdir);
	float2 mr3 = trace_marble(cpos - mrp3, rdir);
	float2 mr4 = trace_marble(cpos - mrp4, rdir);
	float2 mr5 = trace_marble(cpos - mrp5, rdir);
	if( mr1.x<=0 ) mr1.x = 1e9;
	if( mr2.x<=0 ) mr2.x = 1e9;
	if( mr3.x<=0 ) mr3.x = 1e9;
	if( mr4.x<=0 ) mr4.x = 1e9;
	if( mr5.x<=0 ) mr5.x = 1e9;
	
	float4 hm;
	float3 p, n;
	float2 s1 = trace_moon(cpos,rdir,/*min(mr1.x,mr2.x)*/1e9,hm,p);
	float4 ds = 0;
	heat = mystep(m_mist_params.y*m_mist_params.y,m_mist_params.x*m_mist_params.x,s1.y);

	if(s1.x<=0) s1.x = 1e9;
	else		s1.x = dot(p-cpos,rdir);
	float sd = s1.x;
	sd = min(sd, mr1.x);
	sd = min(sd, mr2.x);
	sd = min(sd, mr3.x);
	sd = min(sd, mr4.x);
	sd = min(sd, mr5.x);
	float3 sp = cpos + rdir*sd;
	
	if( sd>=1e9 )
		return 0;

	float ms0 = trace_moon_shadow(sp,ldir);
	float4 ms1 = trace_marble_shadow(sp-mrp1, ldir, ball1_color);
	float4 ms2 = trace_marble_shadow(sp-mrp2, ldir, ball2_color);
	float4 ms3 = trace_marble_shadow(sp-mrp3, ldir, ball3_color);
	float4 ms4 = trace_marble_shadow(sp-mrp4, ldir, ball4_color);
	float4 ms5 = trace_marble_shadow(sp-mrp5, ldir, ball5_color);
	float mse = mystep(0.725*0.725,0.705*0.705,dot(sp,sp));
	float4 trans = 1;
	
		 if( mr1.x==sd )	ds = shade_marble(cpos - mrp1, rdir, ldir, mr1, ball1_color, trans, trans)*lerp(ms0, 1, mse);
	else if( mr2.x==sd )	ds = shade_marble(cpos - mrp2, rdir, ldir, mr2, ball2_color, trans, trans)*lerp(ms0, 1, mse);
	else if( mr3.x==sd )	ds = shade_marble(cpos - mrp3, rdir, ldir, mr3, ball3_color, trans, trans)*lerp(ms0, 1, mse);
	else if( mr4.x==sd )	ds = shade_marble(cpos - mrp4, rdir, ldir, mr4, ball4_color, trans, trans)*lerp(ms0, 1, mse);
	else if( mr5.x==sd )	ds = shade_marble(cpos - mrp5, rdir, ldir, mr5, ball5_color, trans, trans)*lerp(ms0, 1, mse);
	if(s1.x<1e9)
	{
		float4 md = shade_moon(cpos,rdir,ldir,hm,p,(s1.x==sd ? lerp(ms0*ms1*ms2*ms3*ms4*ms5,1,mse) : 1))*trans;
		ds += md;

		float zz = dot(rdir,dir_front)*sd;
		depth_out = (zz-near_plane)/zz;
	}

	return ds;
}




float4 PS_Heat(float2 vpos : vpos) : color
{
	float2 tpos = (vpos+.5)/ssize;
	float heat = tex2D(s_heat,tpos).x;
	float2 r1 = tex2D(s_random,vpos/1024 + float2(0,1)*time*0.021).xy;
	float2 r2 = tex2D(s_random,vpos/1024 + float2(0.866,-0.5)*time*0.022).xy;
	float2 r3 = tex2D(s_random,vpos/1024 + float2(-0.866,0.5)*time*0.023).xy;
	
	tpos *= 500;
	vpos.x += ((r1.x+r2.x+r3.x)*2-3)*heat*m_mist_params.z;
	vpos.y += ((r1.x+r2.x+r3.x)*2-3)*heat*m_mist_params.z;
	
	tpos = (vpos+.5)/ssize;
	return lerp(tex2D(s_screen,tpos),m_mist_color,heat*m_mist_params.w);
}



void VS_Object(
    float3 pos : POSITION,
    float2 texCoord : TEXCOORD0,
    out float4 _pos : POSITION,
    out float2 _texCoord : TEXCOORD0)
{
	float3 wpos;
	float3 cpos = float3(0, 0, -2);
	wpos.x = dot(pos, dir_right);
	wpos.y = dot(pos, dir_up);
	wpos.z = dot(pos, dir_front);
	wpos -= cpos;

	_pos.x = 2*wpos.x*(vpsize.y/vpsize.x)*vzoom;
	_pos.y = 2*-wpos.y*vzoom;
	_pos.z = wpos.z-near_plane;	// Z - near_plane
	_pos.w = wpos.z;			// depth = (Z - near_plane)/Z

    _texCoord = texCoord;
}

float4 PS_Object(
    float2 texCoord : TEXCOORD0,
    out float4 heat : COLOR1) : COLOR0
{
	heat = 0;
	return tex2D(s_object, texCoord);
}

void VS_Object3D(
	float3 pos : POSITION,
	float3 normal : NORMAL,
	out float3 _n : TEXCOORD0,
	out float4 _pos : POSITION)
{
	float3 obj_up = normalize(obj_pos.xyz);
	float3 obj_right = normalize(cross(obj_up,obj_front));

	pos = pos.x*obj_right + pos.y*obj_front + pos.z*obj_up;

	pos *= obj_pos.w;
	pos += obj_pos.xyz;

	float3 wpos;
	float3 cpos = float3(0, 0, -2);
	wpos.x = dot(pos, dir_right);
	wpos.y = dot(pos, dir_up);
	wpos.z = dot(pos, dir_front);
	wpos -= cpos;

	_pos.x = 2*wpos.x*(vpsize.y/vpsize.x)*vzoom;
	_pos.y = 2*-wpos.y*vzoom;
	_pos.z = wpos.z-near_plane;	// Z - near_plane
	_pos.w = wpos.z;			// depth = (Z - near_plane)/Z

	_n.x = dot(normal, dir_right);
	_n.y = dot(normal, dir_up);
	_n.z = dot(normal, dir_front);
}

float4 PS_Object3D(float3 n : TEXCOORD0, out float4 heat : COLOR1) : COLOR0
{
	float3 ldir = normalize(float3(-1,-1,-1));
	n = normalize(n);
	heat = 0;

	float3 eye = float3(0,0,1);
	float d = saturate(dot(n, ldir));
	float s = saturate(dot(ldir,normalize(reflect(eye,n))));
	return d*0.2+pow(s,4)*0.3+pow(s,16)*0.3+pow(s,64)*0.4;
}



technique tech_full {
	pass {
		PixelShader = compile ps_3_0 PS_Raytrace_Full();
		CullMode = NONE;
        AlphaBlendEnable	= FALSE;
    }
}

//technique tech_limited {
//    pass {
//        PixelShader = compile ps_3_0 PS_Raytrace_Limited();
//        CullMode = NONE;
//        AlphaBlendEnable = FALSE;
//    }
//}

technique object {
	pass {
		VertexShader = compile vs_3_0 VS_Object();
		PixelShader = compile ps_3_0 PS_Object();
		CullMode = NONE;
        AlphaTestEnable = TRUE;
        AlphaFunc = GREATER;
        AlphaRef = 128;
    }
}

technique object3d {
	pass {
		VertexShader = compile vs_3_0 VS_Object3D();
		PixelShader = compile ps_3_0 PS_Object3D();
		CullMode = NONE;
	}
}

technique heat {
	pass {
		PixelShader = compile ps_3_0 PS_Heat();
		CullMode = NONE;
        AlphaBlendEnable = FALSE;
    }
}

technique ui {
	pass {
		Lighting			= FALSE;
		AlphaBlendEnable	= TRUE;
		SrcBlend			= SRCALPHA;
		DestBlend			= INVSRCALPHA;
	}
}
