#include "pch.h"
#include "common.h"
#include "ui.h"
#include "TexturePackerSupport.h"
#include "engine.h"

#define FRAME	12


extern int g_skiplevel;



void GameUI::Init()
{
	font.Create("Hevletica",20,0,false,false);
	is_message = false;
	show_game = true;
}

void GameUI::Render()
{
	if(is_message)
		DrawTextBox(message.c_str(),msg_rect,FRAME);
}

void GameUI::DrawTextBox(const char *text,const RECT &box,int frame)
{
	float x1 = (float)box.left;
	float y1 = (float)box.top;
	float x2 = (float)box.right;
	float y2 = (float)box.bottom;
	DWORD c = 0xE0003060;

	struct Vertex {
		float	x, y, z, w;
		DWORD	color;
	} quad[5] = {
		{ x1, y1, 0, 1, c },
		{ x2, y1, 0, 1, c },
		{ x2, y2, 0, 1, c },
		{ x1, y2, 0, 1, c },
		{ x1, y1, 0, 1, c },
	};

	fx.StartTechnique("ui");
	while(fx.StartPass())
	{
		Dev->SetFVF(D3DFVF_XYZRHW|D3DFVF_DIFFUSE);
		Dev->DrawPrimitiveUP(D3DPT_TRIANGLEFAN,2,&quad,sizeof(Vertex));
	}

	for(int i=0;i<5;i++)
		quad[i].color = 0xFFFFFFFF;

	fx.StartTechnique("ui");
	while(fx.StartPass())
	{
		Dev->SetFVF(D3DFVF_XYZRHW|D3DFVF_DIFFUSE);
		Dev->DrawPrimitiveUP(D3DPT_LINESTRIP,4,&quad,sizeof(Vertex));

		RECT rect;
		rect.left = LONG(x1+frame);
		rect.top = LONG(y1+frame);
		rect.right = LONG(x2-frame);
		rect.bottom = LONG(y2-frame);
		font.GetFont()->DrawText(NULL,text,-1,&rect,DT_LEFT|DT_WORDBREAK,0xFFFFFFFF);
	}
}


void GameUI::Tick(float delta)
{
	if(!is_message)
		return;

	while(true)
	{
		int key = Dev.ReadKey();
		if(!key) break;
		if(key<0) continue;

		if(key==VK_F7) { g_skiplevel--; continue; }
		if(key==VK_F8) { g_skiplevel++; continue; }
		if(key=='R')
		{
			ClearMessages();
			game.InitLevel(game.level_id,(INIT_ALL|INIT_RESET_COMPLETED)&~INIT_MESSAGES);
			break;
		}

		is_message = false;
		show_game = true;
		if(msg_queue.size()>0)
		{
			PopMessage(msg_queue[0].text.c_str(),msg_queue[0].width);
			msg_queue.erase(msg_queue.begin());
		}
	}
}

void GameUI::PopMessage(const char *text,int width)
{
	string tt;
	while(*text)
		if(*text=='\\')
		{
			tt.push_back('\n');
			text++;
			while(*text==' ')
				text++;
		}
		else
			tt.push_back(*text++);
	text = tt.c_str();

	if(is_message)
	{
		Message m;
		m.text = text;
		m.width = width;
		msg_queue.push_back(m);
		return;
	}

	if(*text=='*')	show_game=false, text++;
	else			show_game = true;
	while(*text==' ') text++;
	message = text;
	is_message = true;

	msg_rect.top	= 0;
	msg_rect.left	= 0;
	msg_rect.right	= width;
	msg_rect.bottom	= 0;
	font.GetFont()->DrawText(NULL,message.c_str(),-1,&msg_rect,DT_CALCRECT|DT_WORDBREAK,0);

	int sx, sy;
	Dev.GetScreenSize(sx,sy);

	int dx = (sx - (msg_rect.right - msg_rect.left))/2 - msg_rect.left;
	int dy = (sy - (msg_rect.bottom - msg_rect.top))/2 - msg_rect.top;
	msg_rect.left	+= dx - FRAME;
	msg_rect.top	+= dy - FRAME;
	msg_rect.right	+= dx + FRAME;
	msg_rect.bottom	+= dy + FRAME;
}

////////////////////////////////////////////////////////////////////////////////
// GameUI2

//static DevCanvas g_UiCanvas;

GameUI2* g_GameUi;

static DevFont g_UiFont = DevFont("Arial", 24, 0, true, false);

GameUI2::GameUI2()
{
}

void GameUI2::Init()
{
    const char* barFrameName = "laserBlue12.png";
    size_t barFrameIndex = game.m_TexturePackerDoc.FrameFileNameToIndex(barFrameName);
    assert(barFrameIndex != SIZE_MAX);
    game.m_TexturePackerDoc.GetTexCoords(&m_BarTexCoords.x, barFrameIndex);
}

GameUI2::~GameUI2()
{
}

void GameUI2::Update()
{
}

void GameUI2::Render()
{
    for(size_t i = 0, count = game.cameras.size(); i < count; ++i)
    {
        RenderCamera(i);
    }

    // Doesn't work :(
    /*
    g_UiCanvas.SetView(vec2(0.f, 0.f), 2.f);
    g_UiCanvas.Draw(g_TexturePackerTexture)()()(vec2(0.f, 0.f), vec2(1.f, 1.f));
    g_UiCanvas.Flush();
    */
}

void GameUI2::RenderCamera(size_t cameraIndex)
{
    const EngineCamera& cam = game.cameras[cameraIndex];
    const float camLeft = (float)cam.viewport.X;
    const float camTop = (float)cam.viewport.Y;

    float playerEnergy = max(0.f, min(cam.attach_obj.get() ? cam.attach_obj->energy : 0.f, 1.f));
    float homeEnergy = 0.8f;

    const float TEXT_SIZE_X = 148.f;
    const float PERCENT_SIZE_X = 46.f;
    const float MARGIN = 4.f;
    const uint32_t TEXT_COLOR = 0xFF36bbf5;
    char sz[8];

    vec2 basePos = vec2(
        camLeft + 8.f,
        camTop  + 8.f);

    vec2 pos = basePos;
    Dev.AlignPrint(&g_UiFont,
        pos.x, pos.y, 0x00, 0xFFFFFFFF, "Player energy:");
    pos.x += TEXT_SIZE_X + PERCENT_SIZE_X;
    sprintf_s(sz, "%u%%", (uint32_t)(playerEnergy * 100.f + 0.5f));
    Dev.AlignPrint(&g_UiFont,
        pos.x, pos.y, 0x20, 0xFFFFFFFF, sz);
    pos.x += MARGIN;
    RenderBar(pos, playerEnergy);

    basePos.y += 24.f + 4.f;
    pos = basePos;
    Dev.AlignPrint(&g_UiFont,
        basePos.x, basePos.y, 0x00, 0xFFFFFFFF, "Home energy:");
    pos.x += TEXT_SIZE_X + PERCENT_SIZE_X;
    sprintf_s(sz, "%u%%", (uint32_t)(homeEnergy * 100.f + 0.5f));
    Dev.AlignPrint(&g_UiFont,
        basePos.x + TEXT_SIZE_X + PERCENT_SIZE_X, basePos.y, 0x20, 0xFFFFFFFF, sz);
    pos.x += MARGIN;
    RenderBar(pos, homeEnergy);
}

void GameUI2::RenderBar(const vec2& leftTop, float percent)
{
    assert(g_TexturePackerTexture.GetTexture());
    Dev->SetTexture(0, g_TexturePackerTexture.GetTexture());
    Dev->SetPixelShader(nullptr);
    Dev->SetVertexShader(nullptr);

    Dev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
    Dev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
    Dev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_TFACTOR);
    Dev->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
    Dev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
    Dev->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_TFACTOR);

    Dev->SetRenderState(D3DRS_TEXTUREFACTOR, 0xFFFF8080);
    Dev->SetRenderState(D3DRS_ZENABLE, FALSE);

    Dev->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
    Dev->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
    Dev->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
    Dev->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
    Dev->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);

    const vec2 size = vec2(8.f, 24.f);
    const float margin = 2.f;
    const size_t count = 12.f;

    percent = max(0.f, min(percent, 1.f));
    const size_t fullCount = (size_t)(percent * count + 0.5f);
    vec2 pos = leftTop;
    for(size_t i = 0; i < count; ++i)
    {
        const uint32_t color = i < fullCount ?
            0xFFFFFFFF : 0xFF004040;
        Dev->SetRenderState(D3DRS_TEXTUREFACTOR, color);

        Dev.DrawScreenQuadTC(
            pos,
            pos + size,
            vec2(m_BarTexCoords.x, m_BarTexCoords.y),
            vec2(m_BarTexCoords.z, m_BarTexCoords.w),
            0.f);
        pos.x += size.x + margin;
    }
}
